package de.telefonbuch;

import de.telefonbuch.model.Datenbank;
import de.telefonbuch.model.Fehler;
import de.telefonbuch.model.Kontakt;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.TextField;
import java.util.Optional;

/**
 * Der Kontroller für unser Telefonbuch
 */
public class TelefonbuchController {

	private Datenbank db;

	/**
	 * Initialisiert die TreeMap für die Kontakte
	 */
	private Kontakt kontakt = new Kontakt();

	/**
	 * Initialisiert das Fehler Objekt
	 */
	private Fehler fehler = new Fehler();

	@FXML
	private TextField txtVorname;

	@FXML
	private TextField txtNachname;

	@FXML
	private TextField txtStrasse;

	@FXML
	private TextField txtHNr;

	@FXML
	private TextField txtPlz;

	@FXML
	private TextField txtOrt;

	@FXML
	private TextField txtTelefon;

	@FXML
	private Button btnNaviLinks;

	@FXML
	private Button btnNaviRechts;

	@FXML
	private Button btnNeuerEintrag;

	@FXML
	private Button btnLoeschen;

	/**
	 * Action Listener für btnLoeschen
	 */
	@FXML
	void loeschen() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 3);

		deleteKontakt();
	}

	/**
	 * Action Listener für btnWeiter
	 */
	@FXML
	void navWeiter() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		if (db.countKontakte() > 0) {
			kontakt = db.getKontakt(db.getNavi(kontakt.getTelefon())[2]);
			setKontaktToForm(kontakt);
		}
	}

	/**
	 * Action Listener für btnZurueck
	 */
	@FXML
	void navZurueck() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		if (db.countKontakte() > 0) {
			kontakt = db.getKontakt(db.getNavi(kontakt.getTelefon())[0]);
			setKontaktToForm(kontakt);
		}
	}

	/**
	 * Action Listener für btnNeuerEintrag
	 */
	@FXML
	void neuerEintrag() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		neuenKontaktAnlegen();
	}

	/**
	 * Initialisiert die Komponenten
	 */
	@FXML
	void initialize() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		assert txtVorname != null : "fx:id=\"txtVorname\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert txtNachname != null : "fx:id=\"txtNachname\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert txtStrasse != null : "fx:id=\"txtStrasse\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert txtHNr != null : "fx:id=\"txtHNr\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert txtPlz != null : "fx:id=\"txtPlz\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert txtOrt != null : "fx:id=\"txtOrt\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert txtTelefon != null : "fx:id=\"txtTelefon\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert btnNaviLinks != null : "fx:id=\"btnNaviLinks\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert btnNaviRechts != null : "fx:id=\"btnNaviRechts\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert btnNeuerEintrag != null : "fx:id=\"btnNeuerEintrag\" was not injected: check your FXML file 'telefonbuch.fxml'.";
		assert btnLoeschen != null : "fx:id=\"btnLoeschen\" was not injected: check your FXML file 'telefonbuch.fxml'.";

		this.db = new Datenbank();

		int anzahlKontakte = db.countKontakte();

		if (anzahlKontakte > 0) {
			kontakt = db.getKontakt("first");
			if (!kontakt.getTelefon().equals("")) {
				setKontaktToForm(kontakt);
			}
		}
		/* Wenn noch keine Einträge vorhanden sind, die Buttons deaktivieren */
		else {
			kontakt = new Kontakt();
			deaktiviereButtons(true);
		}

		/* Listener für das Vorname Feld */
		txtVorname.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert oder ausgefüllt wurde */
			if (txtVorname.getText() != null && !txtVorname.getText().equals("") && !t1) {
				/* Prüfen ob das Feld den Regeln entspricht */
				if (!kontakt.setVorname(txtVorname.getText())) {
					setFieldError(txtVorname, true);
					fehler.showErrorDialog("error", "", "Der Vorname ist auf 50 Zeichen beschränkt");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {
					saveKontakt();
					setFieldError(txtVorname, false);
				}
			}
		});

		/* Listener für das Nachname Feld */
		txtNachname.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert oder ausgefüllt wurde */
			if (txtNachname.getText() != null && !txtNachname.getText().equals("") && !t1) {
				/* Prüfen ob das Feld den Regeln entspricht */
				if (!kontakt.setNachname(txtNachname.getText())) {
					setFieldError(txtNachname, true);
					fehler.showErrorDialog("error", "", "Der Nachname ist auf 50 Zeichen beschränkt");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {
					saveKontakt();
					setFieldError(txtNachname, false);
				}
			}
		});

		/* Listener für das Straßen Feld */
		txtStrasse.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert oder ausgefüllt wurde */
			if (txtStrasse.getText() != null && !txtStrasse.getText().equals("") && !t1) {
				/* Prüfen ob das Feld den Regeln entspricht */
				if (!kontakt.setStrasse(txtStrasse.getText())) {
					setFieldError(txtStrasse, true);
					fehler.showErrorDialog("error", "", "Die Straße ist auf 50 Zeichen beschränkt");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {
					saveKontakt();
					setFieldError(txtStrasse, false);
				}
			}
		});

		/* Listener für das Hausnummern Feld */
		txtHNr.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert oder ausgefüllt wurde */
			if (txtHNr.getText() != null && !txtHNr.getText().equals("") && !t1) {
				/* Prüfen ob das Feld den Regeln entspricht */
				if (!kontakt.setHausnummer(txtHNr.getText())) {
					setFieldError(txtHNr, true);
					fehler.showErrorDialog("error", "", "Die Hausnummer ist auf 10 Zeichen beschränkt");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {
					saveKontakt();
					setFieldError(txtHNr, false);
				}
			}
		});

		/* Listener für das Plz Feld */
		txtPlz.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert wurde */
			if (!t1) {
				/* Prüfen ob das Feld den Regeln entspricht */
				if (!kontakt.setPlz(txtPlz.getText())) {
					setFieldError(txtPlz, true);
					fehler.showErrorDialog("error", "", "Die Postleitzahl muss 5 Zeichen entsprechen");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {
					saveKontakt();
					setFieldError(txtPlz, false);
				}
			}
		});

		/* Listener für das Ort Feld */
		txtOrt.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert oder ausgefüllt wurde */
			if (txtOrt.getText() != null && !txtOrt.getText().equals("") && !t1) {
				/* Prüfen ob das Feld den Regeln entspricht */
				if (!kontakt.setOrt(txtOrt.getText())) {
					setFieldError(txtOrt, true);
					fehler.showErrorDialog("error", "", "Der Ort ist auf 50 Zeichen beschränkt");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {
					saveKontakt();
					setFieldError(txtOrt, false);
				}
			}
		});

		/* Listener für das Telefon Feld */
		txtTelefon.focusedProperty().addListener((observableValue, aBoolean, t1) -> {
			/* Prüfen ob das Feld verändert wurde */
			if (!t1) {
				if (!kontakt.getTelefon().equals("")) {
					kontakt.setId(kontakt.getTelefon());
				}

				boolean setTelefon = kontakt.setTelefon(txtTelefon.getText());

				/* Wenn das Feld noch leer ist oder nicht den Regeln entspricht, Fehler ausgeben */
				if (!setTelefon || txtTelefon.getText() == null) {
					setFieldError(txtTelefon, true);
					fehler.showErrorDialog("error", "", "Die Telefonnummer muss mindestens 3 und maximal 30 Ziffern lang sein.");
				}
				/* Wenn alles korrekt gelaufen ist, Kontakt speichern und Fehler zurück setzen */
				else {

					setFieldError(txtTelefon, false);

					/* Wenn die Nummer korrekt ausgefüllt wurde und der Kontakt bereits in der DB enthalten war, den Kontakt löschen und neu anlegen */
					if (!txtTelefon.getText().equals("") && txtTelefon.getText() != null) {
						Kontakt tempKontakt = db.getKontakt(txtTelefon.getText());
						if (tempKontakt.getTelefon().equals(txtTelefon.getText())) {
							fehler.showErrorDialog("error", "", "Es gibt bereits einen Eintrag im Adressbuch mit dieser Telefonnummer. Die Telefonnummer muss eindeutig sein.");
						} else {
							saveKontakt();
							kontakt.setId(kontakt.getTelefon());
						}
					}
				}
			}
		});
	}

	/**
	 * Legt ein neues Kontakt Object an
	 */
	private void neuenKontaktAnlegen() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		resetForm();
	}

	/**
	 * Kontakt aus Treemap loeschen
	 */
	private void deleteKontakt() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
		alert.setContentText("Löschen bestätigen");
		alert.setHeaderText(null);
		alert.setContentText("Möchten Sie den aktuellen Eintrag wirklich löschen?");
		Optional<ButtonType> op = alert.showAndWait();

		if (op.isPresent() && op.get() == ButtonType.OK) {
			this.db.deleteKontakt(kontakt.getTelefon());

			if (db.countKontakte() > 0) {
				kontakt = db.getKontakt("first");
				if (!kontakt.getTelefon().isEmpty()) {
					setKontaktToForm(kontakt);
				}
			}
			/* Wenn noch keine Einträge vorhanden sind, die Buttons deaktivieren */
			else {
				kontakt = new Kontakt();
				deaktiviereButtons(true);
				resetForm();
			}
		}
	}

	/**
	 * Kontakt in die Treemap speichern
	 */
	private void saveKontakt() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		if (!kontakt.getTelefon().equals("")) {
			new Fehler().debug("Contact not empty", false);

			db.addKontakt(kontakt);
			kontakt.setId(kontakt.getTelefon());
		}
	}

	/**
	 * Kontrollieren ob alle Felder fehlerfrei sind. Wenn ja, die Buttons wieder aktivieren.
	 */
	private void checkFieldErrors() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		if (!txtVorname.getStyle().equals("-fx-control-inner-background: #ff000040;") &&
				!txtNachname.getStyle().equals("-fx-control-inner-background: #ff000040;") &&
				!txtStrasse.getStyle().equals("-fx-control-inner-background: #ff000040;") &&
				!txtHNr.getStyle().equals("-fx-control-inner-background: #ff000040;") &&
				!txtPlz.getStyle().equals("-fx-control-inner-background: #ff000040;") &&
				!txtOrt.getStyle().equals("-fx-control-inner-background: #ff000040;") &&
				!txtTelefon.getStyle().equals("-fx-control-inner-background: #ff000040;")
		) {
			deaktiviereButtons(false);
		}
	}

	/**
	 * Background der Felder setzen im Fehler Fall sowie deaktivieren der Buttons
	 *
	 * @param field : Textfeld Object
	 * @param set   : true:Fehler false:Kein Fehler
	 */
	private void setFieldError(TextField field, boolean set) {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		if (set) {
			field.setStyle("-fx-control-inner-background: #ff000040;");
			deaktiviereButtons(true);
		} else {
			field.setStyle("-fx-control-inner-background: WHITE;");
			checkFieldErrors();
		}
	}

	/**
	 * Loescht das Formular und erstellt ein neues Kontakt Objekt
	 */
	private void resetForm() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		/* Wenn keine Einträge mehr vorhanden sind, auch die Buttons deaktivieren */
		if (db.countKontakte() == 0) {
			deaktiviereButtons(true);
		}
		kontakt = new Kontakt();
		txtVorname.setText("");
		txtNachname.setText("");
		txtStrasse.setText("");
		txtHNr.setText("");
		txtPlz.setText("");
		txtOrt.setText("");
		txtTelefon.setText("");
	}

	/**
	 * Setzt alle Formular Felder auf die Daten des Objektes
	 *
	 * @param kontakt : Kontakt : Kontakt Objekt
	 */
	private void setKontaktToForm(Kontakt kontakt) {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		txtVorname.setText(kontakt.getVorname());
		txtNachname.setText(kontakt.getNachname());
		txtStrasse.setText(kontakt.getStrasse());
		txtHNr.setText(kontakt.getHausnummer());
		txtPlz.setText(kontakt.getPlz());
		txtOrt.setText(kontakt.getOrt());
		txtTelefon.setText(kontakt.getTelefon());
	}

	/**
	 * Deaktiviert die Buttons
	 *
	 * @param deaktivieren : boolean
	 */
	private void deaktiviereButtons(boolean deaktivieren) {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		if (deaktivieren) {
			btnLoeschen.setDisable(true);
			btnNaviLinks.setDisable(true);
			btnNaviRechts.setDisable(true);
			btnNeuerEintrag.setDisable(true);
		} else if (!txtTelefon.getText().equals("") && txtTelefon.getText() != null) {
			btnLoeschen.setDisable(false);
			btnNaviLinks.setDisable(false);
			btnNaviRechts.setDisable(false);
			btnNeuerEintrag.setDisable(false);
		}
	}
}
